import re, os, sys
from tm_log import *
from tm_globals import trace
import pygame
import pygame.mixer
from pygame.locals import *
import tm_screen

def fit_img_horz(w, h, max_w, max_h):
    new_w = max_w
    new_h = int(h * float(max_w) / float(w))
    return (new_w, new_h)

def fit_img_vert(w, h, max_w, max_h):
    new_h = max_h
    new_w = int(w * float(max_h) / float(h))
    return (new_w, new_h)

def getWavLen(wfile):
    lenproc = subprocess.Popen(["./getlen", wfile],stdout=subprocess.PIPE)
    len = lenproc.communicate()[0]
    return float(len)

def strip_comment(line):

    m = re.compile(r'(.*)#.*').match(line)
    if m:
        line = m.group(1)

    return line

def load_string_from_file(file):

    string = None

    # Load string from file.
    strf_hdl = open(file, 'r')

    string = strf_hdl.readline()
    string = string.rstrip()

    strf_hdl.close()

    string = "asf"

    return string

def load_string_with_id(file, id):

    string = None

    # Load string from file.
    strf_hdl = open(file, 'r')

    while 1:
        l=strf_hdl.readline()
        if not l:
            break
        l = strip_comment(l)
        l = l.rstrip()
        if not l:
            continue
        has_matched = 0
        m = re.compile(r"^%s\s+(.*)" % id).match(l)
        if m:
            string = m.group(1)

    strf_hdl.close()

    return string

class tm_node(object):
    def __init__(self,n):

        self.id = n
        self.recog = {}
        self.input = {}
        self.inputNR = None	# node to go to if No Response (NR)
        self.keys = []
        self.textbox = {}
        self.stills = []
        self.inputs = []
        self.sounds = []
        self.blinks = []


class tm_graph(object):

    def __init__(self):
        self.startnode = None

    def parse(self,f):
        self._nodes_ = {}

        re_node = re.compile(r'^node')
        re_textbox = re.compile(r'^textbox')
        re_audio = re.compile(r'^audio')
        re_still = re.compile(r'^still')
        re_input = re.compile(r'^input')
        re_blink = re.compile(r'^blink')

        fh = open(f,'r')
        while 1:
            y=fh.readline()
            if not y:
                break
            y = strip_comment(y)
            y = y.rstrip()
            if not y:
                continue
            has_matched = 0

            '''--- Node ---'''
            m = re_node.match(y)
            if m:
                nodenum = None

                m = re.compile(r'id="(\w+)"').search(y)
                if m:
                    nodenum = m.group(1)
                else:
                    sys.exit("Graph file error: node definition lacks id.")

                node = tm_node(nodenum)

                m = re.compile(r'recognizer="([\w./]+)"').search(y)
                if m:
                    node.recog['label'] = m.group(1)

                self._nodes_[nodenum] = node

                if self.startnode is None:
                    self.startnode = self._nodes_[nodenum]
                has_matched = 1

            '''--- Audio ---'''
            m = re_audio.match(y)
            if m:
                wavefile = None

                m = re.compile(r'file="([\w./]+)"').search(y)
                if m:
                    wavefile = m.group(1)

                if wavefile == None:
                    sys.exit("Graph file error: node: %s: incomplete audio definition." % self._nodes_[nodenum].id)

                if not os.path.exists(wavefile):                
                    sys.exit("Graph file error: node: %s: bad wavefile path." % self._nodes_[nodenum].id)

                sound = pygame.mixer.Sound(wavefile)
#WINPORT                len   = getWavLen(wavefile)
                len = sound.get_length()

                self._nodes_[nodenum].sounds.append( ( sound, len ) )

                has_matched = 1

            '''--- Inputs ---'''
            m = re_input.match(y)
            if m:
                input = None
                dest = None

                m = re.compile(r'input="(\w+)"').search(y)
                if m:
                    input = m.group(1)

                m = re.compile(r'dest="(\w+)"').search(y)
                if m:
                    dest = m.group(1)

                if input == None or dest == None:
                    sys.exit("Graph file error: node: %s: incomplete input definition" % self._nodes_[nodenum].id)

                if input == "NR":
                    self._nodes_[nodenum].inputNR = dest
                else:
                    self._nodes_[nodenum].inputs.append( (input, dest, -1) )

                has_matched = 1

            '''--- Stills --- '''
            m = re_still.match(y)
            if m:
                still = {}
                id = None
                key = None
                dest = None
                input = None
                still['cell_id'] = None
                still['image'] = None
                still['font'] = None
                still['text'] = None

                m = re.compile(r'cell_id="(\w+)"').search(y)
                if m:
                    still['cell_id'] = m.group(1)

                m = re.compile(r'text_file="([\w./]+)"').search(y)
                if m:
                    file = m.group(1)
                    if file != "last":
                        strf_hdl = open(file, 'r')
                        string = strf_hdl.readline()
                        string = string.rstrip()
                        strf_hdl.close()
                        still['text'] = string
                    else:
                        still['text'] = "last"

                m = re.compile(r'font="([\w./]+)"').search(y)
                if m:
                    still['font'] = m.group(1)

                m = re.compile(r'text="(\w+)"').search(y)
                if m:
                    still['text'] = m.group(1)

                m = re.compile(r'str_id="(\w+)"').search(y)
                if m:
                    id = m.group(1)

                m = re.compile(r'key="(\w+)"').search(y)
                if m:
                    key = m.group(1)

                m = re.compile(r'dest="(\w+)"').search(y)
                if m:
                    dest = m.group(1)

                m = re.compile(r'input="(\w+)"').search(y)
                if m:
                    input = m.group(1)

                if (still['cell_id'] == None):
                    sys.exit("Graph file error: node: %s: incomplete still definition" % self._nodes_[nodenum].id)

                if (still['text'] == None):
                    still['text'] = load_string_with_id(tm_globals.string_file, id)

                m = re.compile(r'image="([\w./]+)"').search(y)
                if m:
                    #trace("loading image for id=%s" % still['cell_id'])
                    jpg = m.group(1)
                    if (jpg == "last"):
                        still['image'] = "last"
                        still['imagerect'] = "last"
                    else:
                        img = pygame.image.load(jpg).convert()
                        assert img

                        # Calculate max image area based upon cell size.
                        rect = tm_screen.get_image_rect(int(still['cell_id']))
                        max_w = rect.width
                        max_h = rect.height

                        #trace("cell image area: %d %d" % (max_w, max_h))

                        # Get actual image size.
                        image_width = img.get_width()
                        image_height = img.get_height()

                        #trace("image size: %d %d" % (image_width, image_height))

                        (new_w, new_h) = fit_img_horz(image_width, image_height, max_w, max_h) 
                        if (new_h > max_h):
                            (new_w, new_h) = fit_img_vert(image_width, image_height, max_w, max_h)

                        #trace("best image size: %d %d" % (new_w, new_h))

                        img = pygame.transform.scale(img, (new_w, new_h))

                        # Center image.
                        left = (max_w - new_w) / 2
                        top = (max_h - new_h) / 2

                        still['image'] = img
                        still['imagerect'] = Rect(left, top, new_w, new_h)

                    self._nodes_[nodenum].stills.append(still)

                if key != None and dest != None:
                    # Add function-key.
                    self._nodes_[nodenum].keys.append((key, dest, int(still['cell_id'])))

                if input != None:
                    self._nodes_[nodenum].inputs.append((input, dest, int(still['cell_id'])))

                has_matched = 1

            '''--- Textbox contents ---'''
            m = re_textbox.match(y)
            if m:
                textbox = {}
                id = None
                textbox['text'] = None
                textbox['font'] = None

                m = re.compile(r'id="(\w+)"').search(y)
                if m:
                    id = m.group(1)

                m = re.compile(r'text="(\w+)"').search(y)
                if m:
                    textbox['text'] = m.group(1)

                m = re.compile(r'font="([\w./]+)"').search(y)
                if m:
                    textbox['font'] = m.group(1)

                if (textbox['text'] == None) and (id == None):
                    sys.exit("Graph file error: node: %s: incomplete textbox definition" % self._nodes_[nodenum].id)

                if (textbox['text'] == None):
                    textbox['text'] = load_string_with_id(tm_globals.string_file, id)

                self._nodes_[nodenum].textbox = textbox

                has_matched = 1

            '''--- Blink contents ---'''
            m = re_blink.match(y)
            if m:
                file = None
                cell_id = None

                m = re.compile(r'file="([\w./]+)"').search(y)
                if m:
                    file = m.group(1)

                m = re.compile(r'cell_id="(\w+)"').search(y)
                if m:
                    cell_id = m.group(1)

                if (file == None) or (cell_id == None):
                    sys.exit("Graph file error: node: %s: incomplete audio definition." % self._nodes_[nodenum].id)

                if not os.path.exists(file):                
                    sys.exit("Graph file error: node: %s: bad blink file path." % self._nodes_[nodenum].id)

                sound = pygame.mixer.Sound(file)
                self._nodes_[nodenum].blinks.append( (sound, int(cell_id) ) )

                has_matched = 1

        if not has_matched:
            trace("Unable to parse: "+y)
        fh.close()

    def nNodes(self):
	return len(self._nodes_)

    def getNodeNames(self):
        return self._nodes_.keys()
	
    def getStartNode(self):
	return self.startnode
	
    def getNode(self, name):
	if self._nodes_.has_key(name):
	    return self._nodes_[name]
	else:
	    return None

if __name__ == '__main__':
    n = tm_graph()
    n.parse('PromptGraph')
    print "Found %d nodes"%n.nNodes()
    n.show()

